# -*- coding: iso-8859-1 -*-

#####################################################################
# Daniel Calderon S.
# CC3501
#####################################################################

# displaylist_01.py
# ---------------
# Dibuja una figuras utilizando listas de opengl
# ---------------

# Implementacin testeada con:
## Python 2.7
## PyOpenGL 3.0.1
## PyGame 1.9.1

#####################################################################

# importar librera grfica
from OpenGL.GL import *
from OpenGL.GLU import *

# importar librera para manejar eventos
import pygame
from pygame.locals import *

from math import pi,sin,cos
from random import uniform

#####################################################################
# Funciones de inicializacin
#####################################################################

def init_pygame((w,h), title=""):
    pygame.init()
    pygame.display.set_mode((w,h), OPENGL|DOUBLEBUF)
    pygame.display.set_caption(title)
	
def init():
    glClearColor(0.0, 0.0, 0.0, 0.0)
    glClearDepth(1.0)
    glDisable(GL_DEPTH_TEST)
    glShadeModel(GL_SMOOTH)
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
    glEnable (GL_BLEND)
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    glHint (GL_LINE_SMOOTH_HINT, GL_NICEST)

def reshape((width, height)):
    if height == 0:
        height = 1
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluOrtho2D(0.0, width, 0.0, height)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
	
def init_opengl((w,h)):
    init()
    reshape((w,h))

#####################################################################
# Funciones de dibujo
#####################################################################

def dibujarLista(rx,ry,px,py,o,rgb,lista):
    glPushMatrix()
    
    glTranslatef(px,py,0.0)
    glScalef(rx,ry,0.0)
    glRotatef(o,0.0,0.0,1.0)
    
    glColor4fv(rgb)
    
    glCallList(lista)
    
    glPopMatrix()

def listaCircunferencia(n):
    
    delta = 2*pi/(n-1)
    
    circ = glGenLists(1)
    glNewList(circ, GL_COMPILE)
    
    glPushMatrix()
    
    glBegin(GL_POLYGON)
    
    glVertex2f(0.0, 0.0)
    
    for i in range(0,n):
        a = delta * i
        glVertex2f( cos(a), sin(a))
    
    glEnd()
    
    glPopMatrix()
    
    glEndList()
    
    return circ
    
def listaEstrellaRandom(n):
    
    delta = 2*pi/(n-1)
    
    circ = glGenLists(1)
    glNewList(circ, GL_COMPILE)
    
    glPushMatrix()
    
    glBegin(GL_POLYGON)
    
    glVertex2f(0.0, 0.0)
    
    for i in range(0,n):
        a = delta * i
        r = uniform(0,1)
        glVertex2f( r*cos(a), r*sin(a))
    
    glEnd()
    
    glPopMatrix()
    
    glEndList()
    
    return circ
	
#####################################################################

w=800 # ancho
h=600 # alto

# inicializando ...
init_pygame((w,h),"Titulo")
init_opengl((w,h))

# definiendo el meteorito
rx = 100
ry = 100
px = w/2
py = h/2
o = 30

# control del movimiento
v = 0.5
left = False
right = False

circ = listaCircunferencia(6)
star = listaEstrellaRandom(100)

rgb1 = [0.7,0.7,0,1]
rgb2 = [1,1,0,1]

# medida de tiempo inicial
t0 = pygame.time.get_ticks()

run = True
while run:
    # 0: CONTROL DEL TIEMPO
	
    # tiempo actual
    t1 = pygame.time.get_ticks()
	
    # diferencial de tiempo asociado a la iteracin
    dt = (t1 - t0)
	
    # actualizar tiempo inicial para siguiente iteracin
    t0 = t1
	
    # 1: MANEJAMOS EVENTOS DE ENTRADA (TECLADO, MOUSE, ETC.)
    for event in pygame.event.get():
        if event.type == QUIT:
            run = False
        if event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                run = False
            if event.key == K_LEFT:
                left = True
            if event.key == K_RIGHT:
                right = True
        if event.type == KEYUP:
            if event.key == K_LEFT:
                left = False
            if event.key == K_RIGHT:
                right = False
				
    # 2: EJECUTAMOS LOGICA DE LA APLICACION
    if right:
        px += v*dt
        
    if left:
        px -= v*dt
	
    # 3: DIBUJAMOS LOS ELEMENTOS
    glClear(GL_COLOR_BUFFER_BIT) # limpia la pantalla
    dibujarLista(rx,ry,px,py,o,rgb1,circ)
    dibujarLista(rx,ry,px,py,o,rgb2,star)
	
    # vuelca el dibujo a la pantalla
    pygame.display.flip()

#####################################################################