# -*- coding: iso-8859-1 -*-

#####################################################################
# Daniel Calderon S.
# CC3501
#####################################################################

# displaylist_textura_01.py
# ---------------
# Dibuja figuras texturizadas utilizando listas de opengl
# ---------------

# Implementacin testeada con:
## Python 2.7
## PyOpenGL 3.0.1
## PyGame 1.9.1

#####################################################################

# importar librera grfica
from OpenGL.GL import *
from OpenGL.GLU import *

from Image import *

# importar librera para manejar eventos
import pygame
from pygame.locals import *

#####################################################################
# Funciones de inicializacin
#####################################################################

def init_pygame((w,h), title=""):
	pygame.init()
	pygame.display.set_mode((w,h), OPENGL|DOUBLEBUF)
	pygame.display.set_caption(title)
	
def init():
    glClearColor(0.0, 0.0, 0.0, 0.0)
    glClearDepth(1.0)
    glDisable(GL_DEPTH_TEST)
    glShadeModel(GL_SMOOTH)
    
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
    glEnable (GL_BLEND)
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    glHint (GL_LINE_SMOOTH_HINT, GL_NICEST)
 
    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR)
    glEnable(GL_TEXTURE_2D)

def reshape((width, height)):
	if height == 0:
		height = 1
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	gluOrtho2D(0.0, width, 0.0, height)
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()
	
def init_opengl((w,h)):
	init()
	reshape((w,h))
 
# METODO PARA CARGAR UNA TEXTURA DESDE UN ARCHIVO
def generarTex(file,repeat):
    #leer el archivo
    image = open(file)
    ix = image.size[0]
    iy = image.size[1]
    image = image.tostring("raw", "RGBX", 0, -1)
    # Crear la textura en opengl    
    texture=glGenTextures(1)
    glBindTexture(GL_TEXTURE_2D, texture)
    glPixelStorei(GL_UNPACK_ALIGNMENT,1)
    
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, ix, iy, 0, GL_RGBA, GL_UNSIGNED_BYTE, image)
    
    if (repeat):
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT)
    else:
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
        
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
    
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE)
    
    return texture

#####################################################################
# Funciones de dibujo
#####################################################################

def dibujarLista(rx,ry,px,py,o,lista):
    glPushMatrix()
    
    glTranslatef(px,py,0.0)
    glScalef(rx,ry,0.0)
    glRotatef(o,0.0,0.0,1.0)
    
    glCallList(lista)
    
    glPopMatrix()

def listaRectanguloTex(texture):
    # rx,ry : tamao en ambas direcciones
    # px,py : posicin
    # o		: orientacin (ngulo en grados)
    
    lista = glGenLists(1)
    glNewList(lista, GL_COMPILE)    
    
    glPushMatrix()
    
    glEnable(GL_TEXTURE_2D)
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE)
    glBindTexture(GL_TEXTURE_2D, texture)
    
    glBegin(GL_POLYGON)			# setea mtodo de dibujo
    
    glTexCoord2f(0.0, 0.0)
    glVertex2f(-0.5,-0.5)
    
    glTexCoord2f(1.0, 0.0)
    glVertex2f(0.5,-0.5)
    
    glTexCoord2f(1.0, 1.0)
    glVertex2f(0.5,0.5)
    
    glTexCoord2f(0.0, 1.0)
    glVertex2f(-0.5,0.5)
    
    glEnd()						# fin del mtodo de dibujo
    
    glDisable(GL_TEXTURE_2D)
    
    glPopMatrix()
    
    glEndList()
    
    return lista

def listaTrianguloTex(texture):
    # rx,ry : tamao en ambas direcciones
    # px,py : posicin
    # o		: orientacin (ngulo en grados)
    
    lista = glGenLists(1)
    glNewList(lista, GL_COMPILE)  
    
    glPushMatrix()
    
    glEnable(GL_TEXTURE_2D)
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE)
    glBindTexture(GL_TEXTURE_2D, texture)
    
    glBegin(GL_TRIANGLES)	# setea mtodo de dibujo
    
    glTexCoord2f(0.0, 0.0)
    glVertex2f(-0.5,-0.5)
    
    glTexCoord2f(1.0, 0.0)
    glVertex2f(0.5,-0.5)
    
    glTexCoord2f(0.5, 1.0)
    glVertex2f(0.0,0.5)
    
#    glTexCoord2f(0.0, 1.0)
#    glVertex2f(-0.5,0.5)
    
    glEnd()						# fin del mtodo de dibujo
    
    glDisable(GL_TEXTURE_2D)
    
    glPopMatrix()
    
    glEndList()
    
    return lista

#####################################################################

w=800 # ancho
h=600 # alto

# inicializando ...
init_pygame((w,h),"Titulo")
init_opengl((w,h))

# definiendo el meteorito
rx = 100
ry = 100
px = w/2 -100
px2 = w/2 +100
py = h/2
o = 0

# control del movimiento
v = 0.5
left = False
right = False

bird_tex = generarTex("bird.png",True)
wall_tex = generarTex("stars.jpg",True)

triangbird = listaTrianguloTex(bird_tex)
squarebird = listaRectanguloTex(bird_tex)

wall = listaRectanguloTex(wall_tex)

# medida de tiempo inicial
t0 = pygame.time.get_ticks()

run = True
while run:
    # 0: CONTROL DEL TIEMPO
	
    # tiempo actual
    t1 = pygame.time.get_ticks()
	
    # diferencial de tiempo asociado a la iteracin
    dt = (t1 - t0)
	
    # actualizar tiempo inicial para siguiente iteracin
    t0 = t1
	
    # 1: MANEJAMOS EVENTOS DE ENTRADA (TECLADO, MOUSE, ETC.)
    for event in pygame.event.get():
        if event.type == QUIT:
            run = False
        if event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                run = False
            if event.key == K_LEFT:
                left = True
            if event.key == K_RIGHT:
                right = True
        if event.type == KEYUP:
            if event.key == K_LEFT:
                left = False
            if event.key == K_RIGHT:
                right = False
				
    # 2: EJECUTAMOS LOGICA DE LA APLICACION
    if right:
        #px += v*dt
        o += v*dt/3.0
    if left:
        #px -= v*dt
        o -= v*dt/3.0
	
    # 3: DIBUJAMOS LOS ELEMENTOS
    glClear(GL_COLOR_BUFFER_BIT) # limpia la pantalla
    
    dibujarLista(w,h,w/2,h/2,0.0,wall)
    dibujarLista(rx,ry,px,py,o,squarebird)
    dibujarLista(rx,ry,px2,py,-o,triangbird)
	
    # pone el dibujo en la pantalla
    pygame.display.flip()
    # ajusta para trabajar a 30 fps.
    # pygame.time.wait(1000/30)

#####################################################################