# -*- coding: utf-8 -*-
"""
Daniel Calderon S.
CC3501

semaforo.py
---------------
Modelacion de un semáforo utilizando una máquina de estados
---------------

Implementación testeada con:
- Python 2.7
- PyOpenGL 3.0.1
- PyGame 1.9.1

"""

# importar librería gráfica
from OpenGL.GL import *
from OpenGL.GLU import *

# importar librería para manejar eventos
import pygame
from pygame.locals import *

from math import pi,sin,cos
#from random import uniform

#####################################################################
# Constantes
#####################################################################

# estados posibles para el semáforo
SEM_RED     = 1
SEM_GREEN   = 2
SEM_YELLOW  = 3

#####################################################################
# Funciones de inicialización
#####################################################################

def init_pygame((w,h), title=""):
    pygame.init()
    pygame.display.set_mode((w,h), OPENGL|DOUBLEBUF)
    pygame.display.set_caption(title)
	
def init():
    glClearColor(0.7, 0.8, 1.0, 1.0)
    glClearDepth(1.0)
    glDisable(GL_DEPTH_TEST)
    glShadeModel(GL_SMOOTH)
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
    glEnable (GL_BLEND)
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    glHint (GL_LINE_SMOOTH_HINT, GL_NICEST)

def reshape((width, height)):
    if height == 0:
        height = 1
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluOrtho2D(0.0, width, 0.0, height)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
	
def init_opengl((w,h)):
    init()
    reshape((w,h))

#####################################################################
# Funciones de dibujo
#####################################################################

def dibujarLista(rx,ry,px,py,o,rgb,lista):
    glPushMatrix()
    
    glTranslatef(px,py,0.0)
    glScalef(rx,ry,0.0)
    glRotatef(o,0.0,0.0,1.0)
    
    glColor4fv(rgb)
    
    glCallList(lista)
    
    glPopMatrix()

def listaCircunferencia(n):
    
    delta = 2*pi/(n)
    
    circ = glGenLists(1)
    glNewList(circ, GL_COMPILE)
    
    glPushMatrix()
    
    glBegin(GL_POLYGON)
    
    glVertex2f(0.0, 0.0)
    
    for i in range(0,n+1):
        a = delta * i
        glVertex2f( 0.5*cos(a), 0.5*sin(a))
    
    glEnd()
    
    glPopMatrix()
    
    glEndList()
    
    return circ
    
#####################################################################
# Clases
#####################################################################

class Semaforo:
    def __init__(self,px,py):
        self.status = SEM_YELLOW
        self.t = 0
        self.t_green    = 5000
        self.t_red      = 5000
        self.t_yellow   = 1000
        
        self.circ = listaCircunferencia(20)

        self.rgb_red    = [1,0,0,1]
        self.rgb_green  = [0,1,0,1]
        self.rgb_yellow = [1,1,0,1]
        self.rgb_gray   = [0.3,0.3,0.3,1]
        
        self.px = px
        self.py = py
        
    def accion(self,dt):
        self.t += dt
        
        # distinto comportamiento segun el estado
        if self.status == SEM_GREEN:
            if self.t > self.t_green:
                self.t = 0
                self.status = SEM_YELLOW
            
        elif self.status == SEM_RED:
            if self.t > self.t_red:
                self.t = 0
                self.status = SEM_GREEN
            
        else: #self.status == SEM_YELLOW:
            if self.t > self.t_yellow:
                self.t = 0
                self.status = SEM_RED
            
        
    def dibujar(self):
        
        #distinta apariencia según el estado
        if self.status == SEM_GREEN:
            dibujarLista(50,50,self.px,self.py + 55 ,0,self.rgb_green,self.circ)
            dibujarLista(50,50,self.px,self.py      ,0,self.rgb_gray,self.circ)
            dibujarLista(50,50,self.px,self.py - 55 ,0,self.rgb_gray,self.circ)
            
        elif self.status == SEM_RED:
            dibujarLista(50,50,self.px,self.py + 55 ,0,self.rgb_gray,self.circ)
            dibujarLista(50,50,self.px,self.py      ,0,self.rgb_gray,self.circ)
            dibujarLista(50,50,self.px,self.py - 55 ,0,self.rgb_red,self.circ)
            
        else: #self.status == SEM_YELLOW:
            dibujarLista(50,50,self.px,self.py + 55 ,0,self.rgb_gray,self.circ)
            dibujarLista(50,50,self.px,self.py      ,0,self.rgb_yellow,self.circ)
            dibujarLista(50,50,self.px,self.py - 55 ,0,self.rgb_gray,self.circ)
        
#####################################################################
# Programa principal
#####################################################################

w=800 # ancho
h=600 # alto

# inicializando ...
init_pygame((w,h),"Titulo")
init_opengl((w,h))

# definiendo el meteorito
rx = 100
ry = 100
px = w/2
py = h/2
o = 30

# control del movimiento
v = 0.5
left = False
right = False

sem = Semaforo(w/2,h/2)

# medida de tiempo inicial
t0 = pygame.time.get_ticks()

run = True
while run:
    # 0: CONTROL DEL TIEMPO
	
    # tiempo actual
    t1 = pygame.time.get_ticks()
	
    # diferencial de tiempo asociado a la iteración
    dt = (t1 - t0)
	
    # actualizar tiempo inicial para siguiente iteración
    t0 = t1
	
    # 1: MANEJAMOS EVENTOS DE ENTRADA (TECLADO, MOUSE, ETC.)
    for event in pygame.event.get():
        if event.type == QUIT:
            run = False
        if event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                run = False
            if event.key == K_LEFT:
                left = True
            if event.key == K_RIGHT:
                right = True
        if event.type == KEYUP:
            if event.key == K_LEFT:
                left = False
            if event.key == K_RIGHT:
                right = False
				
    # 2: EJECUTAMOS LOGICA DE LA APLICACION
    if right:
        sem.px += v*dt
        
    if left:
        sem.px -= v*dt
        
    sem.accion(dt)
	
    # 3: DIBUJAMOS LOS ELEMENTOS
    glClear(GL_COLOR_BUFFER_BIT) # limpia la pantalla
    
    sem.dibujar()
	
    # vuelca el dibujo a la pantalla
    pygame.display.flip()

#####################################################################